/* ordline.r */

/********************************************************************************/

!include <sculptor.h>
!include <printer.h>

/********************************************************************************/

!define _FIND 1
!define _INSERT 2
!define _AMEND 3

!define _LOCK 1
!define _UNLOCK 2

/********************************************************************************/

!file FILE_ORDLINE "../data/ordline"

!record TMP_ORDLINE = FILE_ORDLINE
!record LastRec = FILE_ORDLINE

/********************************************************************************/

!record DispListTable {
	OrderNo,"Order Number",i4
	LineNo,"Line Number",i2
	PartCode,"Part Code",a20
	Qty,"Quantity",i4
}

!record ListTableCellProps {
	cell_type,,i4[4]
	cell_width,,i2[4]
}


/********************************************************************************/

!temp CurrentIndex,,a32
!temp ContinueFind,,i2
!temp Selected,,i2
!temp Inserted,,i2
!temp CurrentOpt,,a20
!temp Found,,i2
!temp LockMain,,i2

/********************************************************************************/

+window wintask at 720, 345 {
	max_width = 480
	max_height = 350
	style = WS_CLOSEBUTTON | WS_MINBUTTON | WS_CAPTION
	event = WinTaskEvent
	title = "ordline"
	data_font = "arial@8"
	label_font = "arial@8"

	button group OptButs {

		+button FindBut at 330, 20 {
			label = "&Find"
			width = 60
			event = FindButEvent
			tooltip = "Find a record by full or partial key values"
		}

		+button NextBut at 330, 45 {
			label = "&Next"
			width = 60
			event = NextButEvent
			tooltip = "Display the next record in ascending key sequence"
		}

		+button PrevBut at 330, 70 {
			label = "&Prev"
			width = 60
			event = PrevButEvent
			tooltip = "Display the previous record in descending key sequence"
		}

		+button MatchBut at 330, 95 {
			label = "&Match"
			width = 60
			event = MatchButEvent
			tooltip = "Display the next record matching the values supplied for the last Find"
		}

		+button ReportBut at 330, 120 {
			label = "&Report"
			width = 60
			event = ReportButEvent
			tooltip = "Print a report on all records in columnar form, or on the selected record"
		}

		+button ListBut at 330, 145 {
			label = "&List"
			width = 60
			event = ListButEvent
			tooltip = "Display all records in table form"
		}

		+button InsertBut at 400, 20 {
			label = "&Insert"
			width = 60
			event = InsertButEvent
			tooltip = "Insert new records"
		}

		+button AmendBut at 400, 45 {
			label = "&Amend"
			width = 60
			event = AmendButEvent
			tooltip = "Amend the currently displayed record"
		}

		+button DeleteBut at 400, 70 {
			label = "&Delete"
			width = 60
			event = DeleteButEvent
			tooltip = "Delete the currently displayed record"
		}

		+button RewindBut at 400, 95 {
			label = "&Rewind"
			width = 60
			event = RewindButEvent
			tooltip = "Rewind the file (or current secondary index) to the start"
		}

		+button WindBut at 400, 120 {
			label = "&Wind"
			width = 60
			event = WindButEvent
			tooltip = "Wind the file (or current secondary index) to the end"
		}

		+button ExitBut at 400, 145 {
			label = "&Exit"
			width = 60
			event = ExitButEvent
			tooltip = "Exit program"
		}
	}

	button group ConfirmButs {
		style = WS_DISABLED


		+button OKBut at 330, 182 {
			label = "&OK"
			width = 60
			event = OKButEvent
			style = WS_DEFAULT_BUTTON
			tooltip = "Confirm the currently displayed data"
		}

		+button CancelBut at 400, 182 {
			label = "&Cancel"
			width = 60
			event = CancelButEvent
			style = WS_CANCEL_BUTTON
			tooltip = "Cancel the current option"
		}
	}

	textbox group KeyFields {
		style = WS_NOTABSTOP

		+textbox OrderNoBox at 78, 20 {
			field = TMP_ORDLINE.OrderNo
			label = "Order Number"
			width = 78
		}

		+textbox LineNoBox at 78, 40 {
			field = TMP_ORDLINE.LineNo
			label = "Line Number"
			width = 48
		}
	}

	textbox group DataFields {
		style = WS_NOTABSTOP

		+textbox PartCodeBox at 78, 70 {
			field = TMP_ORDLINE.PartCode
			label = "Part Code"
			width = 132
		}

		+textbox QtyBox at 78, 90 {
			field = TMP_ORDLINE.Qty
			label = "Quantity"
			width = 78
		}
	}
}

-window winList at 720, 295 {
	max_width = 480
	max_height = 450
	parent = wintask
	style = WS_OWNED
	title = "List of records"

	+table ListTable at 14, 20 {
		source_object = FILE_ORDLINE
		display_object = DispListTable
		rows = 20
		cell_properties = ListTableCellProps
		event = ListTableEvent
	}

	+button OKListBut at 165, 390 {
		label = "&OK"
		width = 60
		event = OKListButEvent
		style = WS_DEFAULT_BUTTON
		tooltip = "Close window, displaying selected record (if any)"
	}

	+button CancelListBut at 240, 390 {
		label = "&Cancel"
		width = 60
		event = CancelListButEvent
		style = WS_CANCEL_BUTTON
		tooltip = "Close window without displaying a record"
	}
}

-window winReport at 810, 427 {
	max_width = 300
	max_height = 185
	parent = wintask
	style = WS_OWNED
	title = "Reports"
	+graphic Graphic1 at 24,12 {
		type = GT_RECTANGLE
		width = 245
		height = 76
		title = "Select report type"
	}

	radio group RepButs {

		+button RepAllBut at 93,32 {
			label = "&All records"
			tooltip = "Print a report of all records in the file, 1 record to a line"
		}

		+button RepCurrentBut at 93,52 {
			label = "&Displayed record"
			tooltip = "Print a report of all fields in the currently selected record"
		}
	}


	+button OKRepBut at 75, 125 {
		label = "&OK"
		width = 60
		event = OKRepButEvent
		style = WS_DEFAULT_BUTTON
		tooltip = "Print selected report"
	}

	+button CancelRepBut at 150, 125 {
		label = "&Cancel"
		width = 60
		event = CancelRepButEvent
		style = WS_CANCEL_BUTTON
		tooltip = "Close window without printing a report"
	}
}

/********************************************************************************/

-window winprompt at 920, 495 {
	parent = wintask
	data_font = "arial@10"
	label_font = "arial@10"
	title = "Sculptor"
}

/********************************************************************************/

	Main()
	exit

/********************************************************************************/

!function Main() {

	Initialise()

MAIN_DIALOG:
	dialog
	return
}

/********************************************************************************/

!function Initialise() {
	openfile FILE_ORDLINE name = "../data/ordline" update
	CurrentIndex = "Main"
	SetListTableCellProps()
	EnableOptButs()
	return
}

/********************************************************************************/

!function WinTaskEvent(EventCode, Object) {

	switch (EventCode) {
	case = EV_CLOSE:
		exit
	}
	return
}

/********************************************************************************/

!function Clear() {
	clear wintask

	clear FILE_ORDLINE
	clear TMP_ORDLINE
	Selected = FALSE
	return
}

/********************************************************************************/

!function Display() {

	TMP_ORDLINE = FILE_ORDLINE
	display wintask
	Selected = TRUE
	LastRec = FILE_ORDLINE
	return
}

/********************************************************************************/

!function FindButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	Clear()
	CurrentOpt = _FIND
	FindMain()
	return
}

/********************************************************************************/

!function FindMain() {

	disable OptButs
	enable ConfirmButs
	enable_tabstop KeyFields
	SetFocusKey1()
	return
}

/********************************************************************************/

!function OKFind() {

	FILE_ORDLINE = TMP_ORDLINE
	findu FILE_ORDLINE nsr = FIND_NSR riu = FIND_RIU traps = FIND_TRAPS

	Found = TRUE
	Display()
	DisableInput()
	setfocus FindBut
	return

FIND_NSR:
	info "No record found"
	SetFocusKey1()
	return

FIND_RIU:
	info "Record is locked by another user"
	SetFocusKey1()
	return

FIND_TRAPS:
	info "Cannot find record (unknown error)"
	SetFocusKey1()
	return
}

/********************************************************************************/

!function NextButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	nextu FILE_ORDLINE nsr = NEXT_NSR riu = NEXT_RIU traps = NEXT_TRAPS

	Display()
	EnableOptButs()
	return

NEXT_NSR:
	info "End of file"
	return

NEXT_RIU:
	info "Next record is locked by another user"
	return

NEXT_TRAPS:
	info "Cannot find next record"
	return
}

/********************************************************************************/

!function PrevButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	prevu FILE_ORDLINE nsr = PREV_NSR riu = PREV_RIU traps = PREV_TRAPS

	Display()
	EnableOptButs()
	return

PREV_NSR:
	info "Beginning of file"
	return

PREV_RIU:
	info "Previous record is locked by another user"
	return

PREV_TRAPS:
	info "Cannot find previous record"
	return
}

/********************************************************************************/

!function MatchButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	matchu FILE_ORDLINE nsr = MATCH_NSR riu = MATCH_RIU traps = MATCH_TRAPS

	Display()
	EnableOptButs()
	return

MATCH_NSR:
	info "No more matching records"
	return

MATCH_RIU:
	info "Next matching record is locked by another user"
	return

MATCH_TRAPS:
	info "Cannot find matching record"
	return
}

/********************************************************************************/

!function ReportButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	show winReport
	switch (Selected) {
		case = FALSE:
			disable RepCurrentBut
			RepAllBut->checked = TRUE
			break

		case = TRUE:
			enable RepCurrentBut
			break
	}

	dialog winReport cancel = RE_CANCEL

RE_CANCEL:
	hide winReport
	return
}

/********************************************************************************/

!function OKRepButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return

	if (RepCurrentBut->checked = TRUE) {
		run report PrintCurrent to "printmanager" ppf = "winpm"
	}

	if (RepAllBut->checked = TRUE) {
		run report PrintAll to "printmanager" ppf = "winpm"
	}

	return CANCEL_DIALOG
}

/********************************************************************************/

!function CancelRepButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	return CANCEL_DIALOG
}

/********************************************************************************/

!function ListButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	set_selected(ListTable, 0, FALSE)
	display ListTable

	show winList
	dialog winList
	hide winList
	return
}

/********************************************************************************/

!function DisplayListRec() {
	if (table_first_selected_key(ListTable) = OKAY) {
		if (ReadRecord(_UNLOCK, "Main") = OKAY) {
			sync_indexes FILE_ORDLINE
			Display()
			EnableOptButs()
		}
	}
	return
}

/********************************************************************************/

!function OKListButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	DisplayListRec()
	return CANCEL_DIALOG
}

/********************************************************************************/

!function CancelListButEvent(EventCode, Object) {

	if (EventCode <> EV_BUTTON_CLICKED) then return
	return CANCEL_DIALOG
}

/********************************************************************************/

!function SetListTableCellProps() {

	ListTableCellProps.cell_type[1] = TCT_READONLY
	ListTableCellProps.cell_type[2] = TCT_READONLY
	ListTableCellProps.cell_type[3] = TCT_READONLY
	ListTableCellProps.cell_type[4] = TCT_READONLY
	create ListTable
	return
}

/********************************************************************************/

!function ListTableEvent(EventCode, Table, Line) {

	if (EventCode = EV_DOUBLE_CLICK) {
		DisplayListRec()
		return CANCEL_DIALOG
	}
	return
}

/********************************************************************************/

!function InsertButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	Clear()
	clear LastRec
	Inserted = FALSE
	CurrentOpt = _INSERT
	EnableInput()
	SetFocusKey1()
	return
}

/********************************************************************************/

!function OKInsert() {

	if (isblank(TMP_ORDLINE)) {
		prompt "", \
			" Insert a record with all null values - are you sure? " \
				button = " Yes " \
				button = " No  ", INSERT_CONTINUE
	}

	CopyToFile()
	insert FILE_ORDLINE re = INSERT_RE riu = INSERT_RIU traps = INSERT_TRAPS

	Selected = TRUE
	Inserted = TRUE
	LastRec = FILE_ORDLINE
	Clear()
	EnableInput()
	SetFocusKey1()
	return

INSERT_RE:
	info "Record already exists with this key"

INSERT_CONTINUE:
	SetFocusKey1()
	return

INSERT_RIU:
	info "File has been locked by another process"
	SetFocusKey1()
	return

INSERT_TRAPS:
	info "Cannot insert record (unknown error)"
	SetFocusKey1()
	return
}

/********************************************************************************/

!function AmendButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	if (ReadRecord(_LOCK, "Main") = ERROR) then return

	CurrentOpt = _AMEND
	EnableInput()

	SetFocusKey1()
	return
}

/********************************************************************************/

!function OKAmend() {

	CopyToFile()
	write FILE_ORDLINE re = AMEND_RE nrs = AMEND_NRS ue = AMEND_UE  traps = AMEND_TRAPS
	Selected = TRUE
	DisableInput()
	setfocus AmendBut
	return

AMEND_RE:
	info "Record already exists with this key"
	SetFocusKey1()
	return

AMEND_NRS:
	info "No record selected"
	Display()
	DisableInput()
	setfocus AmendBut
	return

AMEND_UE:
	info "Update refused - record updated by another user"
	Display()
	DisableInput()
	setfocus AmendBut
	return

AMEND_TRAPS:
	info "Cannot amend record (unknown error)"
	SetFocusKey1()
	return
}

/********************************************************************************/

!function DeleteButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	if (ReadRecord(_LOCK, "Main") = ERROR) then return

	prompt "", \
		" Delete this record? " \
			button = " Yes " \
			button = " No  ", DEL_RETURN

	delete FILE_ORDLINE nrs = DEL_NRS traps = DEL_TRAPS
	Clear()
	EnableOptButs()
	return

DEL_NRS:
	info "No record selected"
	return

DEL_TRAPS:
	info "Unable to delete record"
	return

DEL_RETURN:
	unlock FILE_ORDLINE
	return
}

/********************************************************************************/

!function RewindButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	rewind FILE_ORDLINE

	Clear()

	switch(CurrentIndex) {
		case = "Main":
			caption "File has been rewound to the beginning"
			break

		default:
			caption "Index " + CurrentIndex / " has been rewound to the beginning"
			break
	}

	return
}

/********************************************************************************/

!function WindButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return
	wind FILE_ORDLINE

	Clear()

	switch(CurrentIndex) {
		case = "Main":
			caption "File has been wound to the end"
			break

		default:
			caption "Index " + CurrentIndex / " has been wound to the end"
			break
	}

	return
}

/********************************************************************************/

!function ExitButEvent(EventCode, Object) {
	if (EventCode = EV_BUTTON_CLICKED) then exit
	return
}

/********************************************************************************/

!function OKButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	switch (CurrentOpt) {
		case = _FIND:
			OKFind()
			break

		case = _INSERT:
			OKInsert()
			break

		case = _AMEND:
			OKAmend()
			break
	}
	return
}

/********************************************************************************/

!function CancelButEvent(EventCode, Object) {
	if (EventCode <> EV_BUTTON_CLICKED) then return

	switch (CurrentOpt) {
		case = _FIND:
			Clear()
			setfocus FindBut
			break

		case = _INSERT:
			DisplayLastRecord()
			setfocus InsertBut
			break

		case = _AMEND:
			unlock FILE_ORDLINE
			Display()
			setfocus AmendBut
			break
	}

	DisableInput()
	return
}

/********************************************************************************/

!function EnableOptButs() {
	enable OptButs
	if (Found = FALSE) then disable MatchBut
	if (Selected = FALSE) then disable AmendBut, DeleteBut
	return
}

/********************************************************************************/

!function EnableInput() {
	enable_tabstop KeyFields, DataFields
	disable OptButs
	enable ConfirmButs
	return
}

/********************************************************************************/

!function DisableInput() {
	disable_tabstop KeyFields, DataFields
	disable ConfirmButs
	EnableOptButs()
	return
}

/********************************************************************************/

!function ReadRecord(Mode, Index) {

	if (Index = "") then Index = CurrentIndex
	if (Mode = _LOCK) {
		read FILE_ORDLINE nsr = READ_NSR riu = READ_RIU traps = READ_TRAPS
	}

	if (Mode <> _LOCK) {
		readu FILE_ORDLINE nsr = READ_NSR riu = READ_RIU traps = READ_TRAPS
	}

	return OKAY

READ_NSR:
	info "Record has been deleted by another user"
	Clear()
	return ERROR

READ_RIU:
	info "Record is locked by another user"
	return ERROR

READ_TRAPS:
	info "Cannot reread record (unknown error)"
	return ERROR

}

/********************************************************************************/

!function CopyToFile() {
	FILE_ORDLINE = TMP_ORDLINE
	return
}

/********************************************************************************/

!function DisplayLastRecord() {

	if (Inserted = FALSE) then goto DLR_CLEAR
	FILE_ORDLINE = LastRec
	read FILE_ORDLINE traps = DLR_CLEAR
	Display()
	return

DLR_CLEAR:
	Clear()
	return
}

/********************************************************************************/

!function SetFocusKey1() {
	setfocus OrderNoBox
	return
}

/********************************************************************************/

!report PrintAll {

!drive FILE_ORDLINE

!heading {
	PrintTitle()
	PrintHeadings()
}

!final {
	PrintFinal()
}

	PrintRecord()
	end

}

/********************************************************************************/

!report PrintCurrent {

!temp DataCol,,i2
!temp Element,,i2
!temp PrintHD,,a255

!heading {
	PrintTitle()
}

!title {
	DataCol = 20
	sys.LeftMargin = 10
	sys.LineHeight = sys.CharHeight
	PrintTitle()
	print fit("Order Number:", 19); tab(DataCol); FILE_ORDLINE.OrderNo
	print fit("Line Number:", 19); tab(DataCol); FILE_ORDLINE.LineNo
	print fit("Part Code:", 19); tab(DataCol); FILE_ORDLINE.PartCode
	print fit("Quantity:", 19); tab(DataCol); FILE_ORDLINE.Qty
 	end report
}

	end

}

/********************************************************************************/

!function PrintTitle() {

	print lns(2)
	setfont("Arial", 10, FONT_BOLD)
	print "ordline", sys.Date
	setfont(sys.FontName, sys.FontSize, FONT_NORMAL)
	print
	return
}

/********************************************************************************/

!function PrintHeadings() {

	print
	setfont(sys.FontName, sys.FontSize, FONT_UNDERLINE)
	print tab(1); fit("Order Number", 11);
	print tab(14); fit("Line Number", 6);
	print tab(27); fit("Part Code", 20);
	print tab(40); fit("Quantity", 11);
	print
	setfont(sys.FontName, sys.FontSize, FONT_NORMAL)
	print
	return
}

/********************************************************************************/

!function PrintFinal() {

	print
	print sys.RecordCount: :"s"; " records printed"
	if (Selected = TRUE) {
	FILE_ORDLINE = TMP_ORDLINE
		ReadRecord(_UNLOCK, "Main")
	}
	return
}

/********************************************************************************/

!function PrintRecord() {
	print tab(1); fit(FILE_ORDLINE.OrderNo, 11);
	print tab(14); fit(FILE_ORDLINE.LineNo, 6);
	print tab(27); fit(FILE_ORDLINE.PartCode, 20);
	print tab(40); fit(FILE_ORDLINE.Qty, 11)
	return
}

/********************************************************************************/

!chargrid OFF

/********************************************************************************/
